﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace NetworkCrashSimulator
{
    class Program
    {

        static void Main(string[] args)
        {
            Header();

            CrawlArgs(args);

            Footer(0);
        }

        static void Header()
        {
            Console.WriteLine(string.Empty);
            Version version = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version;
            Console.WriteLine("Network Crash Simulator Version " + version.Major.ToString() + "." + version.Minor.ToString() + "                   Build " + version.Build.ToString() + " Revision " + version.Revision.ToString());
        }

        static void Footer(int exitCode)
        {
            Console.WriteLine(string.Empty);
            Console.WriteLine("This application intend to be a free open source project.");
            Console.WriteLine("This is licenced over a GNU General Public License version 2 (GPLv2)");
            Console.WriteLine("You can visit the application web site on http://ncs.codeplex.com");
            Environment.Exit(exitCode);
        }

        static void CrawlArgs(string[] args)
        {
            if (args.Length > 0)
            {
                if (args[0].Equals("-ListAdapters"))
                {
                    Core.Actions.ListAdaptersConsole();
                }
                else if (args[0].Equals("-Crash"))
                {
                    if (args.Length == 2)
                    {
                        Core.Actions.CrashAdapter(GetAdapterName(args[1]), true);
                    }
                    else
                    {
                        GenerateArgumentError("Invalid arguments where passed.");
                    }
                }
                else if (args[0].Equals("-Repair"))
                {
                    if (args.Length == 2)
                    {
                        Core.Actions.RepairAdapter(GetAdapterName(args[1]));
                    }
                    else
                    {
                        GenerateArgumentError("Invalid arguments where passed.");
                    }

                }
                else if (args[0].Equals("-Timed"))
                {
                    if (args.Length == 3)
                    {

                        List<int> timers = GetTimers(args[2]);
                        string adapterName = GetAdapterName(args[1]);

                        try
                        {
                            Core.Actions.TimedCrash(adapterName, timers[0], timers[1], timers[2]);
                        }
                        catch
                        {
                            GenerateArgumentError("Invalid arguments where passed.");
                        }
                    }
                    else
                    {
                        GenerateArgumentError("Invalid arguments where passed.");
                    }

                }
                else if (args[0].Equals("-Help"))
                {
                    Help.ShowFull();
                }
                else if (args[0].Equals("-GUI"))
                {
                    StartWindowsForm();
                }
                else
                {
                    GenerateArgumentError("Invalid arguments where passed.");
                }
            }
            else
            {
                GenerateArgumentError("No arguments where passed.");
            }
        }

        static string GetAdapterName(string arg)
        {
            string ret = string.Empty; ;

            if (arg.Contains("-Adapter="))
            {
                ret = arg.Replace("-Adapter=", "");
            }
            else
            {
                GenerateArgumentError("The argument -Adapter where not passed.");
            }

            return ret;
        }

        static List<int> GetTimers(string arg)
        {
            List<int> ret = new List<int>();

            if (arg.Contains("-Timers="))
            {
                try
                {
                    string timer = arg.Replace("-Timers=", "");
                    string[] timers = timer.Split(Convert.ToChar(","));

                    foreach (string value in timers)
                    {
                        ret.Add(Convert.ToInt32(value));
                    }
                }
                catch
                {
                    GenerateArgumentError("The argument -Timers is not well formed.");
                }
            }
            else
            {
                GenerateArgumentError("The argument -Timers where not passed.");
            }

            return ret;
        }

        static void GenerateArgumentError(string message)
        {
            Console.WriteLine(string.Empty);
            Console.WriteLine("ERROR: " + message);
            Console.WriteLine(string.Empty);
            Console.WriteLine("Try using the -Help argument to see the Help.");
            Footer(1);
        }

        static void StartWindowsForm()
        {
            Application.EnableVisualStyles();
            Application.Run(new MainForm());
        }
    }
}
